@@ -18,12 +18,14 @@
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
+#include <typeinfo>
+
 #include <QColor>
 #include <QFile>
 #include <QString>
 
 #include <KLocale>
-#include <KDebug>
+#include <KConfigGroup>
 
 #include <element.h>
 #include <entry.h>
@@ -31,16 +33,24 @@
 #include <comment.h>
 #include <preamble.h>
 #include <bibtexentries.h>
+#include <preferences.h>
 
 #include "bibtexfilemodel.h"
 
 static const QRegExp curlyRegExp("[{}]+");
 
+const QString SortFilterBibTeXFileModel::configGroupName = QLatin1String("User Interface");
+
+SortFilterBibTeXFileModel::SortFilterBibTeXFileModel(QObject * parent)
+        : QSortFilterProxyModel(parent), m_internalModel(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")))
+{
+    loadState();
+};
+
 void SortFilterBibTeXFileModel::setSourceModel(QAbstractItemModel *model)
 {
     QSortFilterProxyModel::setSourceModel(model);
     m_internalModel = dynamic_cast<BibTeXFileModel*>(model);
-    m_bibtexFields = BibTeXFields::self();
 }
 
 BibTeXFileModel *SortFilterBibTeXFileModel::bibTeXSourceModel()
@@ -52,40 +62,55 @@ void SortFilterBibTeXFileModel::updateFilter(SortFilterBibTeXFileModel::FilterQu
 {
     m_filterQuery = filterQuery;
     m_filterQuery.field = filterQuery.field.toLower(); /// required for comparison in filter code
-    invalidateFilter();
+    invalidate();
 }
 
 bool SortFilterBibTeXFileModel::lessThan(const QModelIndex & left, const QModelIndex & right) const
 {
     int column = left.column();
-    if (column == right.column() && (m_bibtexFields->at(column).upperCamelCase == QLatin1String("Author") || m_bibtexFields->at(column).upperCamelCase == QLatin1String("Editor"))) {
-        /// special sorting for authors or editors: check all names, compare last and then first names
+    Q_ASSERT(left.column() == right.column()); ///< assume that we only sort by column
+
+    BibTeXFields *bibtexFields = BibTeXFields::self();
+    const FieldDescription &fd = bibtexFields->at(column);
+
+    if (column == right.column() && (fd.upperCamelCase == QLatin1String("Author") || fd.upperCamelCase == QLatin1String("Editor"))) {
+        /// special sorting for authors or editors: check all names,
+        /// compare last and then first names
+
+        /// first, check if two entries (and not e.g. comments) are to be compared
         Entry *entryA = dynamic_cast<Entry*>(m_internalModel->element(left.row()));
         Entry *entryB = dynamic_cast<Entry*>(m_internalModel->element(right.row()));
         if (entryA == NULL || entryB == NULL)
             return QSortFilterProxyModel::lessThan(left, right);
 
-        Value valueA = entryA->value(m_bibtexFields->at(column).upperCamelCase);
-        Value valueB = entryB->value(m_bibtexFields->at(column).upperCamelCase);
+        /// retrieve values of both cells
+        Value valueA = entryA->value(fd.upperCamelCase);
+        Value valueB = entryB->value(fd.upperCamelCase);
         if (valueA.isEmpty())
-            valueA = entryA->value(m_bibtexFields->at(column).upperCamelCaseAlt);
+            valueA = entryA->value(fd.upperCamelCaseAlt);
         if (valueB.isEmpty())
-            valueB = entryB->value(m_bibtexFields->at(column).upperCamelCaseAlt);
+            valueB = entryB->value(fd.upperCamelCaseAlt);
 
+        /// if either value is empty, use default implementation
         if (valueA.isEmpty() || valueB.isEmpty())
             return QSortFilterProxyModel::lessThan(left, right);
 
+        /// compare each person in both values
         for (Value::Iterator itA = valueA.begin(), itB = valueB.begin(); itA != valueA.end() &&  itB != valueB.end(); ++itA, ++itB) {
             Person *personA = dynamic_cast<Person *>(*itA);
             Person *personB = dynamic_cast<Person *>(*itB);
+            /// not a Person object in value? fall back to default implementation
             if (personA == NULL || personB == NULL) return QSortFilterProxyModel::lessThan(left, right);
 
+            /// get both values' next persons' last names for comparison
             QString nameA = personA->lastName().replace(curlyRegExp, "");
             QString nameB = personB->lastName().replace(curlyRegExp, "");
             int cmp = QString::compare(nameA, nameB, Qt::CaseInsensitive);
             if (cmp < 0) return true;
             if (cmp > 0) return false;
 
+            /// if last names were inconclusive ...
+            /// get both values' next persons' first names for comparison
             nameA = personA->firstName().replace(curlyRegExp, "");
             nameB = personB->firstName().replace(curlyRegExp, "");
             cmp = QString::compare(nameA, nameB, Qt::CaseInsensitive);
@@ -95,20 +120,43 @@ bool SortFilterBibTeXFileModel::lessThan(const QModelIndex & left, const QModelI
             // TODO Check for suffix and prefix?
         }
 
+        /// comparison by names did not work (was not conclusive)
+        /// fall back to default implementation
         return QSortFilterProxyModel::lessThan(left, right);
-    } else
+    } else {
+        /// if comparing two numbers, do not perform lexicographical sorting (i.e. 13 < 2),
+        /// but numerical sorting instead (i.e. 13 > 2)
+        const QString textLeft = left.data(Qt::DisplayRole).toString();
+        const QString textRight = right.data(Qt::DisplayRole).toString();
+        bool okLeft = false, okRight = false;
+        int numberLeft = textLeft.toInt(&okLeft);
+        int numberRight = textRight.toInt(&okRight);
+        if (okLeft && okRight)
+            return numberLeft < numberRight;
+
+
+        /// everything else can be sorted by default implementation
+        /// (i.e. alphabetically or lexicographically)
         return QSortFilterProxyModel::lessThan(left, right);
+    }
 }
 
 bool SortFilterBibTeXFileModel::filterAcceptsRow(int source_row, const QModelIndex & source_parent) const
 {
     Q_UNUSED(source_parent)
 
-    if (m_filterQuery.terms.isEmpty()) return true; /// empty filter query
-
     Element *rowElement = m_internalModel->element(source_row);
     Q_ASSERT(rowElement != NULL);
 
+    /// check if showing comments is disabled
+    if (!m_showComments && typeid(*rowElement) == typeid(Comment))
+        return false;
+    /// check if showing macros is disabled
+    if (!m_showMacros && typeid(*rowElement) == typeid(Macro))
+        return false;
+
+    if (m_filterQuery.terms.isEmpty()) return true; /// empty filter query
+
     Entry *entry = dynamic_cast<Entry*>(rowElement);
     if (entry != NULL) {
         /// if current row contains an Entry ...
@@ -122,7 +170,7 @@ bool SortFilterBibTeXFileModel::filterAcceptsRow(int source_row, const QModelInd
             if (m_filterQuery.field.isEmpty() || m_filterQuery.field == it.key().toLower()) {
                 int i = 0;
                 for (QStringList::ConstIterator itsl = m_filterQuery.terms.constBegin(); itsl != m_filterQuery.terms.constEnd(); ++itsl, ++i) {
-                    bool contains = it.value().containsPattern(*itsl);
+                    bool contains = (*itsl).isEmpty() ? true : it.value().containsPattern(*itsl);
                     any |= contains;
                     all[i] |= contains;
                 }
@@ -185,21 +233,36 @@ bool SortFilterBibTeXFileModel::filterAcceptsRow(int source_row, const QModelInd
     return false;
 }
 
+void SortFilterBibTeXFileModel::loadState()
+{
+    KConfigGroup configGroup(config, configGroupName);
+    m_showComments = configGroup.readEntry(BibTeXFileModel::keyShowComments, BibTeXFileModel::defaultShowComments);
+    m_showMacros = configGroup.readEntry(BibTeXFileModel::keyShowMacros, BibTeXFileModel::defaultShowMacros);
+}
 
 
-const QRegExp BibTeXFileModel::whiteSpace = QRegExp("(\\s\\n\\r\\t)+");
+const QString BibTeXFileModel::keyShowComments = QLatin1String("showComments");
+const bool BibTeXFileModel::defaultShowComments = true;
+const QString BibTeXFileModel::keyShowMacros = QLatin1String("showMacros");
+const bool BibTeXFileModel::defaultShowMacros = true;
+
 
 BibTeXFileModel::BibTeXFileModel(QObject * parent)
         : QAbstractTableModel(parent), m_bibtexFile(NULL)
 {
-    m_bibtexFields = BibTeXFields::self();
-// TODO
+    /// load mapping from color value to label
+    KSharedConfigPtr config(KSharedConfig::openConfig(QLatin1String("kbibtexrc")));
+    KConfigGroup configGroup(config, Preferences::groupColor);
+    QStringList colorCodes = configGroup.readEntry(Preferences::keyColorCodes, Preferences::defaultColorCodes);
+    QStringList colorLabels = configGroup.readEntry(Preferences::keyColorLabels, Preferences::defaultcolorLabels);
+    for (QStringList::ConstIterator itc = colorCodes.constBegin(), itl = colorLabels.constBegin(); itc != colorCodes.constEnd() && itl != colorLabels.constEnd(); ++itc, ++itl) {
+        colorToLabel.insert(*itc, *itl);
+    }
 }
 
 BibTeXFileModel::~BibTeXFileModel()
 {
     if (m_bibtexFile != NULL) delete m_bibtexFile;
-// TODO
 }
 
 File *BibTeXFileModel::bibTeXFile()
@@ -210,18 +273,11 @@ File *BibTeXFileModel::bibTeXFile()
 
 void BibTeXFileModel::setBibTeXFile(File *bibtexFile)
 {
+    // FIXME delete old m_bibtexFile before overwriting it?
     m_bibtexFile = bibtexFile;
     reset(); // TODO necessary here?
 }
 
-/*
-QModelIndex BibTeXFileModel::index(int row, int column, const QModelIndex & parent) const
-{
-    Q_UNUSED(parent)
-    return createIndex(row, column, (void*)NULL); // parent == QModelIndex() ? createIndex(row, column, (void*)NULL) : QModelIndex();
-}
-*/
-
 QModelIndex BibTeXFileModel::parent(const QModelIndex & index) const
 {
     Q_UNUSED(index)
@@ -240,7 +296,7 @@ int BibTeXFileModel::rowCount(const QModelIndex & /*parent*/) const
 
 int BibTeXFileModel::columnCount(const QModelIndex & /*parent*/) const
 {
-    return m_bibtexFields->count();
+    return BibTeXFields::self()->count();
 }
 
 QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
@@ -254,22 +310,27 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
         return QVariant();
 
     /// for now, only display data (no editing or icons etc)
-    if (role != Qt::DisplayRole && role != Qt::ToolTipRole && role != Qt::DecorationRole)
+    if (role != Qt::DisplayRole && role != Qt::ToolTipRole && role != Qt::BackgroundRole)
         return QVariant();
 
-    if (index.row() < m_bibtexFile->count() && index.column() < m_bibtexFields->count()) {
-        QString raw = m_bibtexFields->at(index.column()).upperCamelCase;
-        QString rawAlt = m_bibtexFields->at(index.column()).upperCamelCaseAlt;
+    BibTeXFields *bibtexFields = BibTeXFields::self();
+    if (index.row() < m_bibtexFile->count() && index.column() < bibtexFields->count()) {
+        const FieldDescription &fd = bibtexFields->at(index.column());
+        QString raw = fd.upperCamelCase;
+        QString rawAlt = fd.upperCamelCaseAlt;
         Element* element = (*m_bibtexFile)[index.row()];
         Entry* entry = dynamic_cast<Entry*>(element);
 
         /// if BibTeX entry has a "x-color" field, use that color to highlight row
-        if (role == Qt::DecorationRole) {
-            QString color;
-            if (index.column() != 0 || entry == NULL || (color = PlainTextValue::text(entry->value("x-color"), m_bibtexFile)) == "#000000" || color.isEmpty())
+        if (role == Qt::BackgroundRole) {
+            QString colorName;
+            if (entry == NULL || (colorName = PlainTextValue::text(entry->value("x-color"), m_bibtexFile)) == "#000000" || colorName.isEmpty())
                 return QVariant();
-            else
-                return QVariant(QColor(color));
+            else {
+                QColor color(colorName);
+                color.setAlphaF(0.75);
+                return QVariant(color);
+            }
         }
 
         if (entry != NULL) {
@@ -284,14 +345,18 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
                     return QVariant(entry->type());
                 } else
                     return QVariant(label);
+            } else if (raw.toLower() == Entry::ftColor) {
+                QString text = PlainTextValue::text(entry->value(raw), m_bibtexFile);
+                if (text.isEmpty()) return QVariant();
+                QString colorText = colorToLabel[text];
+                if (colorText.isEmpty()) return QVariant(text);
+                return QVariant(colorText);
             } else {
                 if (entry->contains(raw)) {
-                    QString text = PlainTextValue::text(entry->value(raw), m_bibtexFile);
-                    text = text.replace(whiteSpace, " ");
+                    const QString text = PlainTextValue::text(entry->value(raw), m_bibtexFile).simplified();
                     return QVariant(text);
                 } else if (!rawAlt.isNull() && entry->contains(rawAlt)) {
-                    QString text = PlainTextValue::text(entry->value(rawAlt), m_bibtexFile);
-                    text = text.replace(whiteSpace, " ");
+                    const QString text = PlainTextValue::text(entry->value(rawAlt), m_bibtexFile).simplified();
                     return QVariant(text);
                 } else
                     return QVariant();
@@ -304,8 +369,7 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
                 else if (raw == "^type")
                     return QVariant(i18n("Macro"));
                 else if (raw == "Title") {
-                    QString text = PlainTextValue::text(macro->value(), m_bibtexFile);
-                    text = text.replace(whiteSpace, " ");
+                    const QString text = PlainTextValue::text(macro->value(), m_bibtexFile).simplified();
                     return QVariant(text);
                 } else
                     return QVariant();
@@ -315,7 +379,7 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
                     if (raw == "^type")
                         return QVariant(i18n("Comment"));
                     else if (raw == Entry::ftTitle) {
-                        QString text = comment->text().replace(QRegExp("[\\s\\n\\r\\t]+"), " ");
+                        const QString text = comment->text().simplified();
                         return QVariant(text);
                     } else
                         return QVariant();
@@ -325,8 +389,7 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
                         if (raw == "^type")
                             return QVariant(i18n("Preamble"));
                         else if (raw == Entry::ftTitle) {
-                            QString text = PlainTextValue::text(preamble->value(), m_bibtexFile);
-                            text = text.replace(QRegExp("[\\s\\n\\r\\t]+"), " ");
+                            const QString text = PlainTextValue::text(preamble->value(), m_bibtexFile).simplified();
                             return QVariant(text);
                         } else
                             return QVariant();
@@ -341,10 +404,10 @@ QVariant BibTeXFileModel::data(const QModelIndex &index, int role) const
 
 QVariant BibTeXFileModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    if (role != Qt::DisplayRole || orientation != Qt::Horizontal || section < 0 || section >= m_bibtexFields->count())
+    const BibTeXFields *bibtexFields = BibTeXFields::self();
+    if (role != Qt::DisplayRole || orientation != Qt::Horizontal || section < 0 || section >= bibtexFields->count())
         return QVariant();
-
-    return m_bibtexFields->at(section).label;
+    return bibtexFields->at(section).label;
 }
 
 Qt::ItemFlags BibTeXFileModel::flags(const QModelIndex &index) const
