@@ -42,6 +42,8 @@
 #include "elementwidgets.h"
 #include "elementeditor.h"
 
+#define testNullDelete(a) {if ((a)!=NULL) delete (a); (a)=NULL;}
+
 class ElementEditor::ElementEditorPrivate
 {
 private:
@@ -62,6 +64,10 @@ public:
 
     ElementEditorPrivate(Element *m, const File *f, ElementEditor *parent)
             : element(m), file(f), p(parent), previousWidget(NULL), referenceWidget(NULL), sourceWidget(NULL), elementChanged(false), elementUnapplied(false) {
+        internalEntry = NULL;
+        internalMacro = NULL;
+        internalComment = NULL;
+        internalPreamble = NULL;
         createGUI();
     }
 
@@ -175,10 +181,11 @@ public:
             (*it)->setModified(false);
         }
 
-        internalEntry = NULL;
-        internalMacro = NULL;
-        internalComment = NULL;
-        internalPreamble = NULL;
+        testNullDelete(internalEntry);
+        testNullDelete(internalEntry);
+        testNullDelete(internalMacro);
+        testNullDelete(internalComment);
+        testNullDelete(internalPreamble);
         const Entry *e = dynamic_cast<const Entry*>(element);
         if (e != NULL) {
             internalEntry = new Entry(*e);
@@ -224,9 +231,9 @@ public:
             Q_ASSERT(temp != NULL);
 
             previousWidget->apply(temp);
-            if (isSourceWidget) referenceWidget->apply(temp);
+            if (isSourceWidget && referenceWidget != NULL) referenceWidget->apply(temp);
             newWidget->reset(temp);
-            if (dynamic_cast<SourceWidget*>(previousWidget) != NULL)
+            if (referenceWidget != NULL && dynamic_cast<SourceWidget*>(previousWidget) != NULL)
                 referenceWidget->reset(temp);
         }
         previousWidget = newWidget;
@@ -260,7 +267,7 @@ public:
             Value crossRefVal = entry.value(Entry::ftCrossRef);
             if (!crossRefVal.isEmpty() && file != NULL) {
                 crossRefStr = PlainTextValue::text(crossRefVal, file);
-                const Element *crossRefDest = file->containsKey(crossRefStr);
+                const Element *crossRefDest = file->containsKey(crossRefStr, File::etEntry);
                 if (crossRefDest != NULL && typeid(*crossRefDest) == typeid(Entry))
                     dummyFile << new Entry(*dynamic_cast<const Entry*>(crossRefDest));
                 else
@@ -425,6 +432,16 @@ bool ElementEditor::elementUnapplied()
     return d->elementUnapplied;
 }
 
+int ElementEditor::currentTab()
+{
+    return d->tab->currentIndex();
+}
+
+void ElementEditor::setCurrentTab(int tabIndex)
+{
+    d->tab->setCurrentIndex(tabIndex);
+}
+
 void ElementEditor::tabChanged()
 {
     d->switchTo(d->tab->currentWidget());
