@@ -26,6 +26,8 @@
 #include <QFileInfo>
 #include <QDesktopServices>
 #include <QDir>
+#include <QDragEnterEvent>
+#include <QDropEvent>
 
 #include <KDebug>
 #include <KMessageBox>
@@ -33,6 +35,8 @@
 #include <KLocale>
 #include <KUrl>
 #include <KPushButton>
+#include <KSharedConfig>
+#include <KConfigGroup>
 
 #include <fileinfo.h>
 #include <file.h>
@@ -54,14 +58,19 @@ private:
     QSignalMapper *menuTypesSignalMapper;
     KPushButton *buttonOpenUrl;
 
+    KSharedConfigPtr config;
+    const QString configGroupNameGeneral;
+    QString personNameFormatting;
+
 public:
     QMenu *menuTypes;
     KBibTeX::TypeFlag typeFlag;
     KUrl urlToOpen;
     const File *file;
+    QString fieldKey;
 
     FieldLineEditPrivate(KBibTeX::TypeFlag ptf, KBibTeX::TypeFlags tf, FieldLineEdit *p)
-            : parent(p), preferredTypeFlag(ptf), typeFlags(tf), file(NULL) {
+            : parent(p), preferredTypeFlag(ptf), typeFlags(tf), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupNameGeneral(QLatin1String("General")), file(NULL) {
         menuTypes = new QMenu(i18n("Types"), parent);
         menuTypesSignalMapper = new QSignalMapper(parent);
         setupMenu();
@@ -69,6 +78,7 @@ public:
 
         buttonOpenUrl = new KPushButton(KIcon("document-open-remote"), "", parent);
         buttonOpenUrl->setVisible(false);
+        buttonOpenUrl->setProperty("isConst", true);
         parent->appendWidget(buttonOpenUrl);
         connect(buttonOpenUrl, SIGNAL(clicked()), parent, SLOT(slotOpenUrl()));
 
@@ -77,6 +87,15 @@ public:
         Value value;
         typeFlag = determineTypeFlag(value, preferredTypeFlag, typeFlags);
         updateGUI(typeFlag);
+
+        KConfigGroup configGroup(config, configGroupNameGeneral);
+        personNameFormatting = configGroup.readEntry(Person::keyPersonNameFormatting, Person::defaultPersonNameFormatting);
+    }
+
+    ~FieldLineEditPrivate() {
+        delete menuTypes;
+        delete menuTypesSignalMapper;
+        delete buttonOpenUrl;
     }
 
     bool reset(const Value& value) {
@@ -103,13 +122,7 @@ public:
                 } else {
                     const Person *person = dynamic_cast<const Person*>(first);
                     if (typeFlag == KBibTeX::tfPerson && person != NULL) {
-                        text = person->lastName();
-                        QString temp = person->firstName();
-                        if (!temp.isEmpty()) text.prepend(temp + " ");
-                        temp = person->suffix();
-                        if (!temp.isEmpty()) text.append(", " + temp);
-                        temp = person->prefix();
-                        if (!temp.isEmpty()) text.prepend(temp + " ");
+                        text = Person::transcribePersonName(person, personNameFormatting);
                         result = true;
                     } else {
                         const MacroKey *macroKey = dynamic_cast<const MacroKey*>(first);
@@ -364,11 +377,11 @@ public:
 
     void updateURL(const QString &text) {
         QList<KUrl> urls;
-        FileInfo::urlsInText(text, true, file != NULL && file->property(File::Url).value<KUrl>().isValid() ? file->property(File::Url).value<KUrl>().directory() : QString::null, urls);
-        if (urls.isEmpty())
-            urlToOpen = KUrl();
-        else
+        FileInfo::urlsInText(text, true, file != NULL && file->property(File::Url).toUrl().isValid() ? KUrl(file->property(File::Url).toUrl()).directory() : QString::null, urls);
+        if (!urls.isEmpty() && urls.first().isValid())
             urlToOpen = urls.first();
+        else
+            urlToOpen = KUrl();
 
         /// set special "open URL" button visible if URL (or file or DOI) found
         buttonOpenUrl->setVisible(urlToOpen.isValid());
@@ -386,6 +399,13 @@ FieldLineEdit::FieldLineEdit(KBibTeX::TypeFlag preferredTypeFlag, KBibTeX::TypeF
     setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
     setObjectName(QLatin1String("FieldLineEdit"));
     setMenu(d->menuTypes);
+    setChildAcceptDrops(false);
+    setAcceptDrops(true);
+}
+
+FieldLineEdit::~FieldLineEdit()
+{
+    delete d;
 }
 
 bool FieldLineEdit::apply(Value& value) const
@@ -406,7 +426,6 @@ void FieldLineEdit::setReadOnly(bool isReadOnly)
 void FieldLineEdit::slotTypeChanged(int newTypeFlagInt)
 {
     KBibTeX::TypeFlag newTypeFlag = (KBibTeX::TypeFlag)newTypeFlagInt;
-    kDebug() << "new type is " << BibTeXFields::typeFlagToString(newTypeFlag);
 
     Value value;
     d->apply(value);
@@ -423,6 +442,16 @@ void FieldLineEdit::setFile(const File *file)
     d->file = file;
 }
 
+void FieldLineEdit::setElement(const Element *element)
+{
+    Q_UNUSED(element)
+}
+
+void FieldLineEdit::setFieldKey(const QString &fieldKey)
+{
+    d->fieldKey = fieldKey;
+}
+
 void FieldLineEdit::slotOpenUrl()
 {
     d->openUrl();
@@ -432,3 +461,41 @@ void FieldLineEdit::slotTextChanged(const QString &text)
 {
     d->textChanged(text);
 }
+
+void FieldLineEdit::dragEnterEvent(QDragEnterEvent *event)
+{
+    if (event->mimeData()->hasFormat("text/plain") || event->mimeData()->hasFormat("text/x-bibtex"))
+        event->acceptProposedAction();
+}
+
+void FieldLineEdit::dropEvent(QDropEvent *event)
+{
+    const QString clipboardText = event->mimeData()->text();
+    if (clipboardText.isEmpty()) return;
+
+    const File *file = NULL;
+    if (!d->fieldKey.isEmpty() && clipboardText.startsWith("@")) {
+        FileImporterBibTeX importer;
+        file = importer.fromString(clipboardText);
+        const Entry *entry = (file != NULL && file->count() == 1) ? dynamic_cast<const Entry*>(file->first()) : NULL;
+        if (entry != NULL && d->fieldKey == Entry::ftCrossRef) {
+            /// handle drop on crossref line differently (use dropped entry's id)
+            Value v;
+            v.append(new VerbatimText(entry->id()));
+            reset(v);
+            emit textChanged(entry->id());
+            return;
+        } else if (entry != NULL && entry->contains(d->fieldKey)) {
+            /// case for "normal" fields like for journal, pages, ...
+            reset(entry->value(d->fieldKey));
+            emit textChanged(text());
+            return;
+        }
+    }
+
+    if (file == NULL || file->count() == 0) {
+        /// fall-back case: just copy whole text into edit widget
+        setText(clipboardText);
+        emit textChanged(clipboardText);
+    }
+}
