@@ -498,6 +498,7 @@ QString EncoderLaTeX::decode(const QString & text)
             ++it;
             (*cur).append('$').append(*it);
             it = intermediate.erase(it);
+            --it;
         } else
             ++it;
     }
@@ -592,6 +593,7 @@ QString EncoderLaTeX::encode(const QString & text)
             ++it;
             (*cur).append('$').append(*it);
             it = intermediate.erase(it);
+            --it;
         } else
             ++it;
     }
@@ -668,6 +670,23 @@ QString& EncoderLaTeX::decomposedUTF8toLaTeX(QString &text)
     return text;
 }
 
+QString EncoderLaTeX::convertToPlainAscii(const QString &text) const
+{
+    QString internalText = text;
+
+    for (int i = 0; i < modcharmappingdatalatexcount; ++i) {
+        QChar c = QChar(modcharmappingdatalatex[i].unicode);
+        if (internalText.indexOf(c) >= 0)
+            internalText = internalText.replace(c, QString(modcharmappingdatalatex[i].letter));
+    }
+    for (int i = 0; i < commandmappingdatalatexcount; ++i) {
+        QChar c = QChar(commandmappingdatalatex[i].unicode);
+        if (internalText.indexOf(c) >= 0)
+            internalText = internalText.replace(c, QString(commandmappingdatalatex[i].letters));
+    }
+
+    return internalText;
+}
 
 EncoderLaTeX* EncoderLaTeX::currentEncoderLaTeX()
 {
@@ -684,19 +703,3 @@ void EncoderLaTeX::deleteCurrentEncoderLaTeX()
         encoderLaTeX = NULL;
     }
 }
-
-QString& EncoderLaTeX::convertToPlainAscii(QString &text)
-{
-    for (int i = 0; i < modcharmappingdatalatexcount; ++i) {
-        QChar c = QChar(modcharmappingdatalatex[i].unicode);
-        if (text.indexOf(c) >= 0)
-            text = text.replace(c, QString(modcharmappingdatalatex[i].letter));
-    }
-    for (int i = 0; i < commandmappingdatalatexcount; ++i) {
-        QChar c = QChar(commandmappingdatalatex[i].unicode);
-        if (text.indexOf(c) >= 0)
-            text = text.replace(c, QString(commandmappingdatalatex[i].letters));
-    }
-
-    return text;
-}
