@@ -17,6 +17,7 @@
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
+
 #include <typeinfo>
 
 #include <QTextCodec>
@@ -48,45 +49,18 @@
 
 #define encodercheck(encoder, text) ((encoder)?(encoder)->encode((text)):(text))
 
-class ExportWidget : public QWidget
-{
-public:
-    KComboBox *listOfEncodings;
-    static const QString defaultEncoding;
+const QString FileExporterBibTeX::keyEncoding = QLatin1String("encoding");
+const QString FileExporterBibTeX::defaultEncoding = QLatin1String("LaTeX");
+const QString FileExporterBibTeX::keyStringDelimiter = QLatin1String("stringDelimiter");
+const QString FileExporterBibTeX::defaultStringDelimiter = QLatin1String("\"\"");
+const QString FileExporterBibTeX::keyQuoteComment = QLatin1String("quoteComment");
+const FileExporterBibTeX::QuoteComment FileExporterBibTeX::defaultQuoteComment = FileExporterBibTeX::qcNone;
+const QString FileExporterBibTeX::keyKeywordCasing = QLatin1String("keywordCasing");
+const KBibTeX::Casing FileExporterBibTeX::defaultKeywordCasing = KBibTeX::cLowerCase;
+const QString FileExporterBibTeX::keyProtectCasing = QLatin1String("protectCasing");
+const bool FileExporterBibTeX::defaultProtectCasing = true;
 
-    ExportWidget(QWidget *parent)
-            : QWidget(parent) {
-        QFormLayout *layout = new QFormLayout(this);
-        setLayout(layout);
-        setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
-
-        listOfEncodings = new KComboBox(true, this);
-        layout->addRow(i18n("Encoding:"), listOfEncodings);
-
-        listOfEncodings->addItem(QLatin1String("LaTeX"));
-        listOfEncodings->insertSeparator(1);
-        listOfEncodings->addItems(IConvLaTeX::encodings());
-        setProposedEncoding(defaultEncoding);
-    }
-
-    void setProposedEncoding(const QString &proposedEncoding) {
-        QAbstractItemModel *model = listOfEncodings->model();
-        int row = 0;
-        QModelIndex index;
-        const QString lowerPE = proposedEncoding.toLower();
-        while ((index = model->index(row, 0, QModelIndex())) != QModelIndex()) {
-            QString line = model->data(index).toString();
-            if (line.toLower() == lowerPE) {
-                listOfEncodings->lineEdit()->setText(line);
-                listOfEncodings->setCurrentIndex(row);
-                break;
-            }
-            ++row;
-        }
-    }
-};
-
-const QString ExportWidget::defaultEncoding = QLatin1String("LaTeX");
+FileExporterBibTeX *FileExporterBibTeX::staticFileExporterBibTeX = NULL;
 
 class FileExporterBibTeX::FileExporterBibTeXPrivate
 {
@@ -98,24 +72,47 @@ public:
     QChar stringCloseDelimiter;
     KBibTeX::Casing keywordCasing;
     QuoteComment quoteComment;
-    QString encoding;
+    QString encoding, forcedEncoding;
     bool protectCasing;
+    QString personNameFormatting;
     bool cancelFlag;
     IConvLaTeX *iconvLaTeX;
+    KSharedConfigPtr config;
+    const QString configGroupName, configGroupNameGeneral;
 
     FileExporterBibTeXPrivate(FileExporterBibTeX *parent)
-            : p(parent), stringOpenDelimiter(QChar('"')), stringCloseDelimiter(QChar('"')), keywordCasing(KBibTeX::cLowerCase), quoteComment(qcNone), encoding(QLatin1String("latex")), protectCasing(false), cancelFlag(false) {
-        iconvLaTeX = new IConvLaTeX(encoding == QLatin1String("latex") ? QLatin1String("us-ascii") : encoding);
+            : p(parent), cancelFlag(false), iconvLaTeX(NULL), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), configGroupName("FileExporterBibTeX"), configGroupNameGeneral("General") {
+        forcedEncoding = QString::null;
+        loadState();
     }
 
     ~FileExporterBibTeXPrivate() {
         delete iconvLaTeX;
     }
 
+    void loadState() {
+        KConfigGroup configGroup(config, configGroupName);
+        encoding = configGroup.readEntry(p->keyEncoding, p->defaultEncoding);
+        QString stringDelimiter = configGroup.readEntry(p->keyStringDelimiter, p->defaultStringDelimiter);
+        stringOpenDelimiter = stringDelimiter[0];
+        stringCloseDelimiter = stringDelimiter[1];
+        keywordCasing = (KBibTeX::Casing)configGroup.readEntry(p->keyKeywordCasing, (int)p->defaultKeywordCasing);
+        quoteComment = (QuoteComment)configGroup.readEntry(p->keyQuoteComment, (int)p->defaultQuoteComment);
+        protectCasing = configGroup.readEntry(p->keyProtectCasing, p->defaultProtectCasing);
+        personNameFormatting = configGroup.readEntry(Person::keyPersonNameFormatting, "");
+
+        if (personNameFormatting.isEmpty()) {
+            /// no person name formatting is specified for BibTeX, fall back to general setting
+            KConfigGroup configGroupGeneral(config, configGroupNameGeneral);
+            personNameFormatting = configGroupGeneral.readEntry(Person::keyPersonNameFormatting, Person::defaultPersonNameFormatting);
+        }
+    }
+
     bool writeEntry(QIODevice* iodevice, const Entry& entry) {
         BibTeXEntries *be = BibTeXEntries::self();
         BibTeXFields *bf = BibTeXFields::self();
 
+        /// write start of a entry (entry type and id) in plain ASCII
         iodevice->putChar('@');
         iodevice->write(be->format(entry.type(), keywordCasing).toAscii().data());
         iodevice->putChar('{');
@@ -124,9 +121,16 @@ public:
         for (Entry::ConstIterator it = entry.begin(); it != entry.end(); ++it) {
             const QString key = it.key();
             Value value = it.value();
-            QString text = valueToBibTeX(value, key, leUTF8);
-            if (text.isEmpty()) kWarning() << "Value for field " << key << " is empty" << endl;
-            if (protectCasing && dynamic_cast<PlainText*>(value.first()) != NULL && (key == Entry::ftTitle || key == Entry::ftBookTitle || key == Entry::ftSeries))
+            if (value.isEmpty()) continue; ///< ignore empty key-value pairs
+
+            QString text = p->internalValueToBibTeX(value, key, leUTF8);
+            if (text.isEmpty()) {
+                /// ignore empty key-value pairs
+                kWarning() << "Value for field " << key << " is empty" << endl;
+                continue;
+            }
+
+            if (protectCasing && typeid(*value.first()) == typeid(PlainText) && (key == Entry::ftTitle || key == Entry::ftBookTitle || key == Entry::ftSeries))
                 addProtectiveCasing(text);
 
             iodevice->putChar(',');
@@ -149,7 +153,7 @@ public:
     bool writeMacro(QIODevice* iodevice, const Macro& macro) {
         BibTeXEntries *be = BibTeXEntries::self();
 
-        QString text = valueToBibTeX(macro.value(), QString::null, leUTF8);
+        QString text = p->internalValueToBibTeX(macro.value(), QString::null, leUTF8);
         if (protectCasing)
             addProtectiveCasing(text);
 
@@ -205,7 +209,7 @@ public:
         iodevice->putChar('@');
         iodevice->write(be->format(QLatin1String("Preamble"), keywordCasing).toAscii().data());
         iodevice->putChar('{');
-        iodevice->write(iconvLaTeX->encode(valueToBibTeX(preamble.value(), QString::null, leUTF8)));
+        iodevice->write(iconvLaTeX->encode(p->internalValueToBibTeX(preamble.value(), QString::null, leUTF8)));
         iodevice->putChar('}');
         iodevice->putChar('\n');
         iodevice->putChar('\n');
@@ -233,8 +237,42 @@ public:
         if (addBrackets)
             text.insert(1, '{').insert(text.length(), '}');
     }
+
+    void applyEncoding(QString& encoding) {
+        encoding = encoding.isEmpty() ? QLatin1String("latex") : encoding.toLower();
+        delete iconvLaTeX;
+        iconvLaTeX = new IConvLaTeX(encoding == QLatin1String("latex") ? QLatin1String("us-ascii") : encoding);
+    }
+
+    bool requiresPersonQuoting(const QString &text, bool isLastName) {
+        if (isLastName && !text.contains(" "))
+            /** Last name contains NO spaces, no quoting necessary */
+            return false;
+        else if (!isLastName && !text.contains(" and "))
+            /** First name contains no " and " no quoting necessary */
+            return false;
+        else if (isLastName && !text.isEmpty() && text[0].isLower())
+            /** Last name starts with lower-case character (von, van, de, ...) */
+            // FIXME does not work yet
+            return false;
+        else if (text[0] != '{' || text[text.length() - 1] != '}')
+            /** as either last name contains spaces or first name contains " and " and there is no protective quoting yet, there must be a protective quoting added */
+            return true;
+
+        int bracketCounter = 0;
+        for (int i = text.length() - 1; i >= 0; --i) {
+            if (text[i] == '{')
+                ++bracketCounter;
+            else if (text[i] == '}')
+                --bracketCounter;
+            if (bracketCounter == 0 && i > 0)
+                return true;
+        }
+        return false;
+    }
 };
 
+
 FileExporterBibTeX::FileExporterBibTeX()
         : FileExporter(), d(new FileExporterBibTeXPrivate(this))
 {
@@ -243,43 +281,12 @@ FileExporterBibTeX::FileExporterBibTeX()
 
 FileExporterBibTeX::~FileExporterBibTeX()
 {
-// nothing
+    delete d;
 }
 
-void FileExporterBibTeX::setEncoding(const QString& encoding)
+void FileExporterBibTeX::setEncoding(const QString &encoding)
 {
-    QString normalizedEncoding = encoding.isNull() ? QLatin1String("latex") : encoding.toLower();
-    if (normalizedEncoding != d->encoding) {
-        d->encoding = normalizedEncoding;
-        delete d->iconvLaTeX;
-        d->iconvLaTeX = new IConvLaTeX(d->encoding == QLatin1String("latex") ?  QLatin1String("us-ascii") : encoding);
-    }
-}
-
-QString FileExporterBibTeX::encoding() const
-{
-    return d->encoding;
-}
-
-void FileExporterBibTeX::setStringDelimiters(const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter)
-{
-    d->stringOpenDelimiter = stringOpenDelimiter;
-    d->stringCloseDelimiter = stringCloseDelimiter;
-}
-
-void FileExporterBibTeX::setKeywordCasing(KBibTeX::Casing keywordCasing)
-{
-    d->keywordCasing = keywordCasing;
-}
-
-void FileExporterBibTeX::setQuoteComment(QuoteComment quoteComment)
-{
-    d->quoteComment = quoteComment;
-}
-
-void FileExporterBibTeX::setProtectCasing(bool protectCasing)
-{
-    d->protectCasing = protectCasing;
+    d->forcedEncoding = encoding;
 }
 
 bool FileExporterBibTeX::save(QIODevice* iodevice, const File* bibtexfile, QStringList * /*errorLog*/)
@@ -314,7 +321,7 @@ bool FileExporterBibTeX::save(QIODevice* iodevice, const File* bibtexfile, QStri
                     Comment *comment = dynamic_cast<Comment*>(*it);
                     QString commentText = QString::null;
                     /** check if this file requests a special encoding */
-                    if (comment == NULL || !comment->text().startsWith("x-kbibtex-encoding="))
+                    if (comment == NULL || !comment->text().startsWith("x-kbibtex-"))
                         remainingList.append(*it);
                 }
             }
@@ -324,15 +331,33 @@ bool FileExporterBibTeX::save(QIODevice* iodevice, const File* bibtexfile, QStri
     int totalElements = (int) bibtexfile->count();
     int currentPos = 0;
 
-    QString effectiveEncoding = ExportWidget::defaultEncoding;
-    if (!d->encoding.isEmpty())
-        effectiveEncoding = d->encoding;
+    loadState();
     if (bibtexfile->hasProperty(File::Encoding))
-        effectiveEncoding = bibtexfile->property(File::Encoding).toString();
-    setEncoding(effectiveEncoding);
+        d->encoding = bibtexfile->property(File::Encoding).toString();
+    if (!d->forcedEncoding.isEmpty())
+        d->encoding = d->forcedEncoding;
+    d->applyEncoding(d->encoding);
+    if (bibtexfile->hasProperty(File::StringDelimiter)) {
+        QString stringDelimiter = bibtexfile->property(File::StringDelimiter).toString();
+        d->stringOpenDelimiter = stringDelimiter[0];
+        d->stringCloseDelimiter = stringDelimiter[1];
+    }
+    if (bibtexfile->hasProperty(File::QuoteComment))
+        d->quoteComment = (QuoteComment)bibtexfile->property(File::QuoteComment).toInt();
+    if (bibtexfile->hasProperty(File::KeywordCasing))
+        d->keywordCasing = (KBibTeX::Casing)bibtexfile->property(File::KeywordCasing).toInt();
+    if (bibtexfile->hasProperty(File::ProtectCasing))
+        d->protectCasing = bibtexfile->property(File::ProtectCasing).toBool();
+    if (bibtexfile->hasProperty(File::NameFormatting)) {
+        /// if the user set "use global default", this property is an empty string
+        /// in this case, keep default value
+        const QString buffer = bibtexfile->property(File::NameFormatting).toString();
+        d->personNameFormatting = buffer.isEmpty() ? d->personNameFormatting : buffer;
+    }
 
-    if (effectiveEncoding != QLatin1String("latex"))
-        parameterCommentsList << new Comment("x-kbibtex-encoding=" + effectiveEncoding, true);
+    if (d->encoding != QLatin1String("latex"))
+        parameterCommentsList << new Comment("x-kbibtex-encoding=" + d->encoding, true);
+    parameterCommentsList << new Comment("x-kbibtex-personnameformatting=" + d->personNameFormatting, true);
 
     /** before anything else, write parameter comments */
     for (QList<Comment*>::ConstIterator it = parameterCommentsList.begin(); it != parameterCommentsList.end() && result && !d->cancelFlag; it++) {
@@ -377,6 +402,11 @@ bool FileExporterBibTeX::save(QIODevice* iodevice, const Element* element, QStri
 {
     bool result = false;
 
+    loadState();
+    if (!d->forcedEncoding.isEmpty())
+        d->encoding = d->forcedEncoding;
+    d->applyEncoding(d->encoding);
+
     const Entry *entry = dynamic_cast<const Entry*>(element);
     if (entry != NULL)
         result |= d->writeEntry(iodevice, *entry);
@@ -404,31 +434,16 @@ void FileExporterBibTeX::cancel()
     d->cancelFlag = true;
 }
 
-void FileExporterBibTeX::showExportDialog(QWidget *parent, File *bibtexfile) const
+QString FileExporterBibTeX::valueToBibTeX(const Value& value, const QString& key, UseLaTeXEncoding useLaTeXEncoding)
 {
-    Q_ASSERT(bibtexfile != NULL);
-
-    KDialog dialog(parent);
-    dialog.setButtons(KDialog::Ok);
-
-    /// default encoding is LaTeX
-    QString proposedEncoding = ExportWidget::defaultEncoding;
-    /// check encoding set to this exporter
-    if (!d->encoding.isEmpty())
-        proposedEncoding = d->encoding;
-    /// encoding as stored in the File has highest precendence
-    if (bibtexfile->hasProperty(File::Encoding))
-        proposedEncoding = bibtexfile->property(File::Encoding).toString();
-
-    ExportWidget exportWidget(&dialog);
-    exportWidget.setProposedEncoding(proposedEncoding);
-    dialog.setMainWidget(&exportWidget);
-
-    if (dialog.exec() == QDialog::Accepted)
-        bibtexfile->setProperty(File::Encoding, exportWidget.listOfEncodings->lineEdit()->text());
+    if (staticFileExporterBibTeX == NULL)
+        staticFileExporterBibTeX = new FileExporterBibTeX();
+    else
+        staticFileExporterBibTeX->loadState();
+    return staticFileExporterBibTeX->internalValueToBibTeX(value, key, useLaTeXEncoding);
 }
 
-QString FileExporterBibTeX::valueToBibTeX(const Value& value, const QString& key, UseLaTeXEncoding useLaTeXEncoding)
+QString FileExporterBibTeX::internalValueToBibTeX(const Value& value, const QString& key, UseLaTeXEncoding useLaTeXEncoding)
 {
     if (value.isEmpty())
         return "";
@@ -437,11 +452,13 @@ QString FileExporterBibTeX::valueToBibTeX(const Value& value, const QString& key
 
     QString result = "";
     bool isOpen = false;
+    /// variable to memorize which closing delimiter to use
+    QChar stringCloseDelimiter = d->stringCloseDelimiter;
     const ValueItem* prev = NULL;
     for (QList<ValueItem*>::ConstIterator it = value.begin(); it != value.end(); ++it) {
         const MacroKey *macroKey = dynamic_cast<const MacroKey*>(*it);
         if (macroKey != NULL) {
-            if (isOpen) result.append('}');
+            if (isOpen) result.append(stringCloseDelimiter);
             isOpen = false;
             if (!result.isEmpty()) result.append(" # ");
             result.append(macroKey->text());
@@ -449,82 +466,145 @@ QString FileExporterBibTeX::valueToBibTeX(const Value& value, const QString& key
         } else {
             const PlainText *plainText = dynamic_cast<const PlainText*>(*it);
             if (plainText != NULL) {
+                QString textBody = encodercheck(encoder, escapeLaTeXChars(plainText->text()));
                 if (!isOpen) {
                     if (!result.isEmpty()) result.append(" # ");
-                    result.append('{');
+                    if (textBody.contains("\"")) {
+                        /// fall back to {...} delimiters if text contains quotation marks
+                        result.append("{");
+                        stringCloseDelimiter = '}';
+                    } else {
+                        result.append(d->stringOpenDelimiter);
+                        stringCloseDelimiter = d->stringCloseDelimiter;
+                    }
                 } else if (prev != NULL && typeid(*prev) == typeid(PlainText))
                     result.append(' ');
                 else if (prev != NULL && typeid(*prev) == typeid(Person)) {
                     /// handle "et al." i.e. "and others"
                     result.append(" and ");
-                } else
-                    result.append("} # {");
+                } else {
+                    result.append(stringCloseDelimiter).append(" # ");
+
+                    if (textBody.contains("\"")) {
+                        /// fall back to {...} delimiters if text contains quotation marks
+                        result.append("{");
+                        stringCloseDelimiter = '}';
+                    } else {
+                        result.append(d->stringOpenDelimiter);
+                        stringCloseDelimiter = d->stringCloseDelimiter;
+                    }
+                }
                 isOpen = true;
-                result.append(encodercheck(encoder, escapeLaTeXChars(plainText->text())));
+                result.append(textBody);
                 prev = plainText;
             } else {
                 const VerbatimText *verbatimText = dynamic_cast<const VerbatimText*>(*it);
                 if (verbatimText != NULL) {
+                    QString textBody = verbatimText->text();
                     if (!isOpen) {
                         if (!result.isEmpty()) result.append(" # ");
-                        result.append('{');
+                        if (textBody.contains("\"")) {
+                            /// fall back to {...} delimiters if text contains quotation marks
+                            result.append("{");
+                            stringCloseDelimiter = '}';
+                        } else {
+                            result.append(d->stringOpenDelimiter);
+                            stringCloseDelimiter = d->stringCloseDelimiter;
+                        }
                     } else if (prev != NULL && typeid(*prev) == typeid(VerbatimText)) {
                         if (key.toLower().startsWith(Entry::ftUrl) || key.toLower().startsWith(Entry::ftLocalFile) || key.toLower().startsWith(Entry::ftDOI))
                             result.append("; ");
                         else
                             result.append(' ');
-                    } else
-                        result.append("} # {");
+                    } else {
+                        result.append(stringCloseDelimiter).append(" # ");
+
+                        if (textBody.contains("\"")) {
+                            /// fall back to {...} delimiters if text contains quotation marks
+                            result.append("{");
+                            stringCloseDelimiter = '}';
+                        } else {
+                            result.append(d->stringOpenDelimiter);
+                            stringCloseDelimiter = d->stringCloseDelimiter;
+                        }
+                    }
                     isOpen = true;
-                    result.append(verbatimText->text());
+                    result.append(textBody);
                     prev = verbatimText;
                 } else {
                     const Person *person = dynamic_cast<const Person*>(*it);
                     if (person != NULL) {
-                        if (!isOpen) {
-                            if (!result.isEmpty()) result.append(" # ");
-                            result.append('{');
-                        } else if (prev != NULL && typeid(*prev) == typeid(Person))
-                            result.append(" and ");
-                        else
-                            result.append("} # {");
-                        isOpen = true;
+                        QString firstName = person->firstName();
+                        if (!firstName.isEmpty() && d->requiresPersonQuoting(firstName, false))
+                            firstName = firstName.prepend("{").append("}");
 
-                        QString thisName = "";
-                        QString v = person->firstName();
-                        if (!v.isEmpty()) {
-                            bool requiresQuoting = requiresPersonQuoting(v, false);
-                            if (requiresQuoting) thisName.append("{");
-                            thisName.append(v);
-                            if (requiresQuoting) thisName.append("}");
-                            thisName.append(" ");
-                        }
-
-                        v = person->lastName();
-                        if (!v.isEmpty()) {
-                            bool requiresQuoting = requiresPersonQuoting(v, false);
-                            if (requiresQuoting) thisName.append("{");
-                            thisName.append(v);
-                            if (requiresQuoting) thisName.append("}");
-                        }
+                        QString lastName = person->lastName();
+                        if (!lastName.isEmpty() && d->requiresPersonQuoting(lastName, true))
+                            lastName = lastName.prepend("{").append("}");
 
                         // TODO: Prefix and suffix
 
+                        QString thisName = Person::transcribePersonName(d->personNameFormatting, firstName, lastName);
+
+                        if (!isOpen) {
+                            if (!result.isEmpty()) result.append(" # ");
+                            if (thisName.contains("\"")) {
+                                /// fall back to {...} delimiters if text contains quotation marks
+                                result.append("{");
+                                stringCloseDelimiter = '}';
+                            } else {
+                                result.append(d->stringOpenDelimiter);
+                                stringCloseDelimiter = d->stringCloseDelimiter;
+                            }
+                        } else if (prev != NULL && typeid(*prev) == typeid(Person))
+                            result.append(" and ");
+                        else {
+                            result.append(stringCloseDelimiter).append(" # ");
+
+                            if (thisName.contains("\"")) {
+                                /// fall back to {...} delimiters if text contains quotation marks
+                                result.append("{");
+                                stringCloseDelimiter = '}';
+                            } else {
+                                result.append(d->stringOpenDelimiter);
+                                stringCloseDelimiter = d->stringCloseDelimiter;
+                            }
+                        }
+                        isOpen = true;
+
                         result.append(encodercheck(encoder, escapeLaTeXChars(thisName)));
                         prev = person;
                     } else {
                         const Keyword *keyword = dynamic_cast<const Keyword*>(*it);
                         if (keyword != NULL) {
+                            QString textBody = encodercheck(encoder, escapeLaTeXChars(keyword->text()));
                             if (!isOpen) {
                                 if (!result.isEmpty()) result.append(" # ");
-                                result.append('{');
+                                if (textBody.contains("\"")) {
+                                    /// fall back to {...} delimiters if text contains quotation marks
+                                    result.append("{");
+                                    stringCloseDelimiter = '}';
+                                } else {
+                                    result.append(d->stringOpenDelimiter);
+                                    stringCloseDelimiter = d->stringCloseDelimiter;
+                                }
                             } else if (prev != NULL && typeid(*prev) == typeid(Keyword))
                                 result.append("; ");
-                            else
-                                result.append("} # {");
+                            else {
+                                result.append(stringCloseDelimiter).append(" # ");
+
+                                if (textBody.contains("\"")) {
+                                    /// fall back to {...} delimiters if text contains quotation marks
+                                    result.append("{");
+                                    stringCloseDelimiter = '}';
+                                } else {
+                                    result.append(d->stringOpenDelimiter);
+                                    stringCloseDelimiter = d->stringCloseDelimiter;
+                                }
+                            }
                             isOpen = true;
 
-                            result.append(encodercheck(encoder, escapeLaTeXChars(keyword->text())));
+                            result.append(textBody);
                             prev = keyword;
                         }
                     }
@@ -534,7 +614,7 @@ QString FileExporterBibTeX::valueToBibTeX(const Value& value, const QString& key
         prev = *it;
     }
 
-    if (isOpen) result.append('}');
+    if (isOpen) result.append(stringCloseDelimiter);
     return result;
 }
 
@@ -552,35 +632,47 @@ QString FileExporterBibTeX::elementToString(const Element* element)
 
 QString FileExporterBibTeX::escapeLaTeXChars(const QString &text)
 {
+    /// Regular expression to match dollar signs that are not escaped (i.e. not \$).
+    /// Store character in front of dollar sign in cap(1).
+    const QRegExp regExpMathSeparator = QRegExp(QLatin1String("(^|[^\\\\])\\$"));
+    /// Regular expression for characters to escape for LaTeX
+    const QRegExp regExpEscape("[^\\\\][&#_%]");
+    /// Status whether in math mode or not (as determined by dollar signs)
+    bool inMathMode = false;
+    /// Resulting text
     QString result = text;
 
-    const QRegExp regExpEscape("[^\\\\][&#_%]");
-    int p = -1;
-    while ((p = regExpEscape.indexIn(result, p + 1)) != -1)
-        result = result.left(p + 1) + '\\' + result.mid(p + 1);
+    int m1 = -1, m2 = -1;
+    /// For each dollar sign (= switch into or out of math mode) ...
+    while ((m1 = regExpMathSeparator.indexIn(result, m2 + 1)) >= 0) {
+        /// Compensate for character in front of dollar sign
+        int cl = regExpMathSeparator.cap(1).length();
+        m1 += cl;
+
+        if (!inMathMode) {
+            /// If not in math mode, make special characters LaTeX-safe
+            int p = m2;
+            while ((p = regExpEscape.indexIn(result, p + 1)) >= 0 && p < m1) {
+                result = result.left(p + 1) + '\\' + result.mid(p + 1);
+                ++m1;
+            }
+        }
+        /// Toggle math mode
+        inMathMode = !inMathMode;
+
+        m2 = m1;
+    }
+
+    if (m1 == -1 && !inMathMode) {
+        int p = m2;
+        while ((p = regExpEscape.indexIn(result, p + 1)) >= 0)
+            result = result.left(p + 1) + '\\' + result.mid(p + 1);
+    }
+
     return result;
 }
 
-bool FileExporterBibTeX::requiresPersonQuoting(const QString &text, bool isLastName)
+void FileExporterBibTeX::loadState()
 {
-    if (isLastName && !text.contains(" "))
-        /** Last name contains NO spaces, no quoting necessary */
-        return FALSE;
-    else if (!isLastName && !text.contains(" and "))
-        /** First name contains no " and " no quoting necessary */
-        return FALSE;
-    else if (text[0] != '{' || text[text.length()-1] != '}')
-        /** as either last name contains spaces or first name contains " and " and there is no protective quoting yet, there must be a protective quoting added */
-        return TRUE;
-
-    int bracketCounter = 0;
-    for (int i = text.length() - 1; i >= 0; --i) {
-        if (text[i] == '{')
-            ++bracketCounter;
-        else if (text[i] == '}')
-            --bracketCounter;
-        if (bracketCounter == 0 && i > 0)
-            return TRUE;
-    }
-    return false;
+    d->loadState();
 }
