@@ -18,6 +18,8 @@
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
+#include <typeinfo>
+
 #include <QTextCodec>
 #include <QIODevice>
 #include <QRegExp>
@@ -42,10 +44,10 @@
 #include "fileimporterbibtex.h"
 
 const QString extraAlphaNumChars = QString("?'`-_:.+/$\\\"&");
-const QRegExp htmlRegExp = QRegExp("</?(a|pre)[^>]*>", Qt::CaseInsensitive);
+const QRegExp htmlRegExp = QRegExp("</?(a|pre|p|br|span|i|b|italic)\\b[^>]*>", Qt::CaseInsensitive);
 
-FileImporterBibTeX::FileImporterBibTeX(const QString& encoding, bool ignoreComments, KBibTeX::Casing keywordCasing)
-        : FileImporter(), m_cancelFlag(false), m_lineNo(1), m_textStream(NULL), m_currentChar(' '), m_ignoreComments(ignoreComments), m_encoding(encoding.toLower()), m_newEncoding(QString::null), m_keywordCasing(keywordCasing)
+FileImporterBibTeX::FileImporterBibTeX(bool ignoreComments, KBibTeX::Casing keywordCasing)
+        : FileImporter(), m_cancelFlag(false), m_lineNo(1), m_textStream(NULL), m_currentChar(' '), m_ignoreComments(ignoreComments), m_keywordCasing(keywordCasing)
 {
     // nothing
 }
@@ -58,21 +60,19 @@ File* FileImporterBibTeX::load(QIODevice *iodevice)
 {
     m_cancelFlag = false;
 
+    File *result = new File();
+
     m_textStreamLastPos = 0;
     m_textStream = new QTextStream(iodevice);
-    m_newEncoding = m_encoding;
-    m_textStream->setCodec(m_encoding == "latex" ? "UTF-8" : m_encoding.toAscii());
+    m_textStream->setCodec("us-ascii"); ///< unless we learn something else, assume 7-bit US-ASCII
     QString rawText = "";
     while (!m_textStream->atEnd()) {
         QString line = m_textStream->readLine();
-        bool skipline = evaluateParameterComments(m_textStream, line.toLower());
+        bool skipline = evaluateParameterComments(m_textStream, line.toLower(), result);
         if (!skipline)
             rawText.append(line).append("\n");
     }
 
-    File *result = new File();
-    result->setProperty(m_newEncoding == "latex" ? "latex" : File::Encoding, m_textStream->codec()->name());
-
     delete m_textStream;
 
     /** Remove HTML code from the input source */
@@ -92,8 +92,7 @@ File* FileImporterBibTeX::load(QIODevice *iodevice)
         Element * element = nextElement();
 
         if (element != NULL) {
-            Comment *comment = dynamic_cast<Comment*>(element);
-            if (!m_ignoreComments || comment == NULL)
+            if (!m_ignoreComments || typeid(*element) != typeid(Comment))
                 result->append(element);
             else
                 delete element;
@@ -114,7 +113,7 @@ File* FileImporterBibTeX::load(QIODevice *iodevice)
 
 bool FileImporterBibTeX::guessCanDecode(const QString & rawText)
 {
-    QString text = EncoderLaTeX::currentEncoderLaTeX() ->decode(rawText);
+    QString text = EncoderLaTeX::currentEncoderLaTeX()->decode(rawText);
     return text.indexOf(QRegExp("@\\w+\\{.+\\}")) >= 0;
 }
 
@@ -207,7 +206,7 @@ Macro *FileImporterBibTeX::readMacroElement()
     Macro *macro = new Macro(key);
     do {
         bool isStringKey = false;
-        QString text = readString(isStringKey).replace(QRegExp("\\s+"), " ");
+        QString text = readString(isStringKey).simplified();
         if (isStringKey)
             macro->value().append(new MacroKey(text));
         else
@@ -233,7 +232,7 @@ Preamble *FileImporterBibTeX::readPreambleElement()
     Preamble *preamble = new Preamble();
     do {
         bool isStringKey = FALSE;
-        QString text = readString(isStringKey).replace(QRegExp("\\s+"), " ");
+        QString text = readString(isStringKey).simplified();
         if (isStringKey)
             preamble->value().append(new MacroKey(text));
         else
@@ -293,10 +292,6 @@ Entry *FileImporterBibTeX::readEntryElement(const QString& typeString)
             return NULL;
         }
 
-        /// some sources such as ACM's Digital Library use "issue" instead of "number" -> fix that
-        if (keyName.toLower() == QLatin1String("issue"))
-            keyName = QLatin1String("number"); // FIXME use constants here or even better code?
-
         Value value;
 
         /** check for duplicate fields */
@@ -463,7 +458,7 @@ QString FileImporterBibTeX::readLine()
     return result;
 }
 
-QString FileImporterBibTeX::readBracketString(const QChar openingBracket)
+QString FileImporterBibTeX::readBracketString(const QChar openingBracket) ///< do not use reference on QChar here!
 {
     QString result;
     QChar closingBracket = '}';
@@ -497,16 +492,15 @@ FileImporterBibTeX::Token FileImporterBibTeX::readValue(Value& value, const QStr
 
     do {
         bool isStringKey = false;
-        QString text = readString(isStringKey).trimmed();
+        QString text = readString(isStringKey).simplified();
         /// for all entries except for abstracts ...
         if (iKey != Entry::ftAbstract) {
             /// ... remove redundant spaces including newlines
-            text = text.replace(QRegExp("\\s+"), " ");
+            text = text.simplified();
         }
         /// abstracts will keep their formatting (regarding line breaks)
         /// as requested by Thomas Jensch via mail (20 October 2010)
 
-        // FIXME: comparisons should be made case-insensitive
         if (iKey == Entry::ftAuthor || iKey == Entry::ftEditor) {
             if (isStringKey)
                 value.append(new MacroKey(text));
@@ -536,7 +530,7 @@ FileImporterBibTeX::Token FileImporterBibTeX::readValue(Value& value, const QStr
                 value.append(new MacroKey(text));
             else
                 value.append(new PlainText(text));
-        } else if (iKey.startsWith(Entry::ftUrl) || iKey.startsWith(Entry::ftLocalFile) || iKey.compare(QLatin1String("ee"), Qt::CaseInsensitive) == 0 || iKey.compare(QLatin1String("biburl"), Qt::CaseInsensitive) == 0) {
+        } else if ((iKey.startsWith(Entry::ftUrl) && !iKey.startsWith(Entry::ftUrlDate)) || iKey.startsWith(Entry::ftLocalFile) || iKey.compare(QLatin1String("ee"), Qt::CaseInsensitive) == 0 || iKey.compare(QLatin1String("biburl"), Qt::CaseInsensitive) == 0) {
             if (isStringKey)
                 value.append(new MacroKey(text));
             else {
@@ -664,20 +658,20 @@ void FileImporterBibTeX::splitPersonList(const QString& text, QStringList &resul
 Person *FileImporterBibTeX::splitName(const QString& text)
 {
     QStringList segments;
-    bool containsComma = splitName(text, segments);
+    CommaContainment commaContainment = splitName(text, segments);
     QString firstName = "";
     QString lastName = "";
 
     if (segments.isEmpty())
         return NULL;
 
-    if (!containsComma) {
+    if (commaContainment == ccNoComma) {
         /** PubMed uses a special writing style for names, where the last name is followed by single capital letter,
           * each being the first letter of each first name
           * So, check how many single capital letters are at the end of the given segment list */
         int singleCapitalLettersCounter = 0;
         int p = segments.count() - 1;
-        while (segments[p].length() == 1 && segments[p].compare(segments[p].toUpper()) == 0) {
+        while (p >= 0 && segments[p].length() == 1 && segments[p][0].isUpper()) {
             --p;
             ++singleCapitalLettersCounter;
         }
@@ -732,10 +726,10 @@ Person *FileImporterBibTeX::splitName(const QString& text)
 }
 
 /** Splits a name into single words. If the name's text was reversed (Last, First), the result will be true and the comma will be added to segments. Otherwise the functions result will be false. This function respects protecting {...}. */
-bool FileImporterBibTeX::splitName(const QString& text, QStringList& segments)
+FileImporterBibTeX::CommaContainment FileImporterBibTeX::splitName(const QString& text, QStringList& segments)
 {
     int bracketCounter = 0;
-    bool result = FALSE;
+    CommaContainment result = ccNoComma;
     QString buffer = "";
 
     for (int pos = 0; pos < text.length(); ++pos) {
@@ -755,7 +749,7 @@ bool FileImporterBibTeX::splitName(const QString& text, QStringList& segments)
                 buffer = "";
             }
             segments.append(",");
-            result = TRUE;
+            result = ccContainsComma;
         } else
             buffer.append(text[pos]);
     }
@@ -766,14 +760,18 @@ bool FileImporterBibTeX::splitName(const QString& text, QStringList& segments)
     return result;
 }
 
-bool FileImporterBibTeX::evaluateParameterComments(QTextStream *textStream, const QString &line)
+bool FileImporterBibTeX::evaluateParameterComments(QTextStream *textStream, const QString &line, File *file)
 {
     /** check if this file requests a special encoding */
     if (line.startsWith("@comment{x-kbibtex-encoding=") && line.endsWith("}")) {
-        m_newEncoding = line.mid(28, line.length() - 29).toLower();
-        kDebug() << "x-kbibtex-encoding=" << m_newEncoding;
-        textStream->setCodec(m_newEncoding == "latex" ? "UTF-8" : m_newEncoding.toAscii());
-        kDebug() <<  "m_textStream->codec()=" << m_textStream->codec()->name();
+        QString encoding = line.mid(28, line.length() - 29).toLower();
+        textStream->setCodec(encoding == "latex" ? "UTF-8" : encoding.toAscii());
+        encoding = textStream->codec()->name();
+        file->setProperty(File::Encoding, encoding);
+        return true;
+    } else if (line.startsWith("@comment{x-kbibtex-personnameformatting=") && line.endsWith("}")) {
+        QString personNameFormatting = line.mid(40, line.length() - 41);
+        file->setProperty(File::NameFormatting, personNameFormatting);
         return true;
     }
 
