@@ -18,16 +18,19 @@
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
+#include <typeinfo>
+
 #include <QLabel>
 #include <QFile>
+#include <QFileInfo>
 #include <QApplication>
 #include <QLayout>
 #include <QKeyEvent>
 #include <QSignalMapper>
 #include <QDesktopServices>
 
+#include <KFileDialog>
 #include <KDebug>
-#include <KEncodingFileDialog>
 #include <KMessageBox>
 #include <KLocale>
 #include <KAction>
@@ -47,6 +50,7 @@
 #include <fileimporterris.h>
 #include <fileexporterris.h>
 #include <fileimporterpdf.h>
+#include <fileexporterps.h>
 #include <fileexporterpdf.h>
 #include <fileexporterrtf.h>
 #include <fileexporterbibtex2html.h>
@@ -57,6 +61,10 @@
 #include <preamble.h>
 #include <comment.h>
 #include <filterbar.h>
+#include <findduplicatesui.h>
+#include <lyx.h>
+#include <preferences/settingscolorlabelwidget.h>
+#include <preferences/settingsfileexporterbibtexwidget.h>
 
 #include <valuelistmodel.h>
 #include <clipboard.h>
@@ -85,26 +93,32 @@ public:
     QMenu *viewDocumentMenu;
     QSignalMapper *signalMapperViewDocument;
     bool isSaveAsOperation;
+    LyX *lyx;
+    ColorLabelContextMenu *colorLabelContextMenu;
 
     KBibTeXPartPrivate(KBibTeXPart *parent)
-            : p(parent), sortFilterProxyModel(NULL), signalMapperNewElement(new QSignalMapper(parent)), viewDocumentMenu(new QMenu(i18n("View Document"), parent->widget())), signalMapperViewDocument(new QSignalMapper(parent)), isSaveAsOperation(false) {
+            : p(parent), model(NULL), sortFilterProxyModel(NULL), signalMapperNewElement(new QSignalMapper(parent)), viewDocumentMenu(new QMenu(i18n("View Document"), parent->widget())), signalMapperViewDocument(new QSignalMapper(parent)), isSaveAsOperation(false) {
         connect(signalMapperViewDocument, SIGNAL(mapped(QObject*)), p, SLOT(elementViewDocumentMenu(QObject*)));
     }
 
+    ~KBibTeXPartPrivate() {
+        delete model;
+        delete signalMapperNewElement;
+        delete viewDocumentMenu;
+        delete signalMapperViewDocument;
+    }
+
     FileImporter *fileImporterFactory(const KUrl& url) {
         QString ending = url.path().toLower();
         int p = ending.lastIndexOf(".");
         ending = ending.mid(p + 1);
 
         if (ending == "pdf") {
-            kDebug() << "Selecting FileImporterPDF" << endl;
             return new FileImporterPDF();
         } else if (ending == "ris") {
-            kDebug() << "Selecting FileImporterRIS" << endl;
             return new FileImporterRIS();
         } else {
-            kDebug() << "Selecting FileImporterBibTeX" << endl;
-            return new FileImporterBibTeX("latex", false);
+            return new FileImporterBibTeX(false);
         }
     }
 
@@ -114,25 +128,20 @@ public:
         ending = ending.mid(p + 1);
 
         if (ending == "html") {
-            kDebug() << "Selecting FileExporterXSLT" << endl;
             return new FileExporterXSLT();
         } else if (ending == "xml") {
-            kDebug() << "Selecting FileExporterXML" << endl;
             return new FileExporterXML();
         } else if (ending == "ris") {
-            kDebug() << "Selecting FileExporterRIS" << endl;
             return new FileExporterRIS();
         } else if (ending == "pdf") {
-            kDebug() << "Selecting FileExporterPDF" << endl;
             return new FileExporterPDF();
+        } else if (ending == "ps") {
+            return new FileExporterPS();
         } else if (ending == "rtf") {
-            kDebug() << "Selecting FileExporterRTF" << endl;
             return new FileExporterRTF();
         } else if (ending == "html" || ending == "html") {
-            kDebug() << "Selecting FileExporterBibTeX2HTML" << endl;
             return new FileExporterBibTeX2HTML();
         } else {
-            kDebug() << "Selecting FileExporterBibTeX" << endl;
             return new FileExporterBibTeX();
         }
     }
@@ -158,7 +167,6 @@ public:
         sortFilterProxyModel->setSourceModel(model);
         editor->setModel(sortFilterProxyModel);
         connect(filterBar, SIGNAL(filterChanged(SortFilterBibTeXFileModel::FilterQuery)), sortFilterProxyModel, SLOT(updateFilter(SortFilterBibTeXFileModel::FilterQuery)));
-        //connect(editor->valueListWidget(), SIGNAL(filterChanged(SortFilterBibTeXFileModel::FilterQuery)), filterBar, SLOT(setFilter(SortFilterBibTeXFileModel::FilterQuery))); //FIXME
     }
 
     void makeBackup(const KUrl &url) const {
@@ -197,16 +205,17 @@ public:
     }
 
     KUrl getSaveFilename(bool mustBeImportable = true) {
-        QString startDir = QString();// QLatin1String(":save"); // FIXME: Does not work yet
+        QString startDir = p->url().isValid() ? p->url().path() : QLatin1String("kfiledialog:///opensave");
         QString supportedMimeTypes = QLatin1String("text/x-bibtex application/xml application/x-research-info-systems");
         if (!mustBeImportable && FileExporterToolchain::kpsewhich(QLatin1String("embedfile.sty")))
             supportedMimeTypes += QLatin1String(" application/pdf");
+        if (!mustBeImportable && FileExporterToolchain::which(QLatin1String("dvips")))
+            supportedMimeTypes += QLatin1String(" application/postscript");
         if (!mustBeImportable)
             supportedMimeTypes += QLatin1String(" text/html");
         if (!mustBeImportable && FileExporterToolchain::which(QLatin1String("latex2rtf")))
             supportedMimeTypes += QLatin1String(" application/rtf");
 
-
         return KFileDialog::getSaveUrl(startDir, supportedMimeTypes, p->widget());
     }
 
@@ -226,14 +235,20 @@ public:
         SortFilterBibTeXFileModel *model = dynamic_cast<SortFilterBibTeXFileModel *>(editor->model());
         Q_ASSERT(model != NULL);
         FileExporter *exporter = fileExporterFactory(url);
-        FileExporterBibTeX *exporterBibTeX = dynamic_cast<FileExporterBibTeX*>(exporter);
 
         if (isSaveAsOperation) {
             /// only show export dialog at SaveAs or SaveCopyAs operations
-            exporter->showExportDialog(p->widget(), model->bibTeXSourceModel()->bibTeXFile());
+
+            if (typeid(*exporter) == typeid(FileExporterBibTeX)) {
+                KDialog dlg(p->widget());
+                File *file = model->bibTeXSourceModel()->bibTeXFile();
+                SettingsFileExporterBibTeXWidget settingsWidget(file, &dlg);
+                dlg.setMainWidget(&settingsWidget);
+                dlg.setButtons(KDialog::Ok);
+                dlg.exec();
+                settingsWidget.saveProperties(file);
+            }
         }
-        if (exporterBibTeX != NULL)
-            exporterBibTeX->setEncoding(model->bibTeXSourceModel()->bibTeXFile()->property(File::Encoding).toString());
 
         qApp->setOverrideCursor(Qt::WaitCursor);
 
@@ -258,14 +273,6 @@ public:
         qApp->restoreOverrideCursor();
         if (!result)
             KMessageBox::error(p->widget(), i18n("Saving the bibliography to file \"%1\" failed.", url.pathOrUrl()), i18n("Saving bibliography failed"));
-        else {
-            File *file = model->bibTeXSourceModel()->bibTeXFile();
-            /// store new URL in BibTeX File object
-            file->setProperty(File::Url, url);
-            /// store encoding in BibTeX File object
-            if (exporterBibTeX != NULL)
-                file->setProperty(File::Encoding, exporterBibTeX->encoding());
-        }
 
         delete exporter;
         return result;
@@ -279,22 +286,22 @@ public:
         if (!info.exists())
             return true;
 
-        return KMessageBox::Cancel != KMessageBox::warningContinueCancel(parent,     i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?",  info.fileName()),
-                i18n("Overwrite File?"), KStandardGuiItem::overwrite(),    KStandardGuiItem::cancel(), QString(), KMessageBox::Notify | KMessageBox::Dangerous);
+        return KMessageBox::Cancel != KMessageBox::warningContinueCancel(parent, i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?", info.fileName()), i18n("Overwrite File?"), KStandardGuiItem::overwrite(), KStandardGuiItem::cancel(), QString(), KMessageBox::Notify | KMessageBox::Dangerous);
     }
 
     int updateViewDocumentMenu() {
         viewDocumentMenu->clear();
         int result = 0;
 
-        Entry *entry = dynamic_cast<Entry*>(editor->currentElement());
+        const Entry *entry = dynamic_cast<const Entry*>(editor->currentElement());
         if (entry != NULL) {
             QList<KUrl> urlList = FileInfo::entryUrls(entry, editor->bibTeXModel()->bibTeXFile()->property(File::Url).value<KUrl>());
             if (!urlList.isEmpty()) {
                 for (QList<KUrl>::ConstIterator it = urlList.constBegin(); it != urlList.constEnd(); ++it) {
                     // FIXME: the signal mapper will fill up with mappings, as they are never removed
-                    KAction *action = new KAction((*it).pathOrUrl(), p); // TODO beautify (icon, url)
+                    KAction *action = new KAction(KIcon(KMimeType::iconNameForUrl(*it)), (*it).pathOrUrl(), p);
                     action->setData((*it).pathOrUrl());
+                    action->setToolTip((*it).prettyUrl());
                     connect(action, SIGNAL(triggered()), signalMapperViewDocument, SLOT(map()));
                     signalMapperViewDocument->setMapping(action, action);
                     viewDocumentMenu->addAction(action);
@@ -314,7 +321,7 @@ KBibTeXPart::KBibTeXPart(QWidget *parentWidget, QObject *parent, bool browserVie
     setObjectName("KBibTeXPart::KBibTeXPart");
 
     // TODO Setup view
-    d->editor = new BibTeXEditor(parentWidget);
+    d->editor = new BibTeXEditor(QLatin1String("Main"), parentWidget);
     d->editor->setReadOnly(!isReadWrite());
     setWidget(d->editor);
 
@@ -335,7 +342,7 @@ KBibTeXPart::KBibTeXPart(QWidget *parentWidget, QObject *parent, bool browserVie
 
 KBibTeXPart::~KBibTeXPart()
 {
-    // nothing
+    delete d;
 }
 
 void KBibTeXPart::setModified(bool modified)
@@ -355,6 +362,7 @@ void KBibTeXPart::setupActions(bool /*browserViewWanted FIXME*/)
     connect(saveCopyAsAction, SIGNAL(triggered()), this, SLOT(documentSaveCopyAs()));
 
     d->filterBar = new FilterBar(0);
+    d->editor->setFilterBar(d->filterBar);
     KAction *filterWidgetAction = new KAction(i18n("Filter"), this);
     actionCollection()->addAction("toolbar_filter_widget", filterWidgetAction);
     filterWidgetAction->setShortcut(Qt::CTRL + Qt::Key_F);
@@ -406,26 +414,36 @@ void KBibTeXPart::setupActions(bool /*browserViewWanted FIXME*/)
     actionCollection()->addAction(QLatin1String("edit_copy_references"),  d->editCopyReferencesAction);
     d->editPasteAction = actionCollection()->addAction(KStandardAction::Paste, clipboard, SLOT(paste()));
 
+    /// build context menu for central BibTeX file view
     d->editor->setContextMenuPolicy(Qt::ActionsContextMenu);
-    d->editor->insertAction(NULL, d->elementEditAction);
-    d->editor->insertAction(NULL, d->elementViewDocumentAction);
+    d->editor->addAction(d->elementEditAction);
+    d->editor->addAction(d->elementViewDocumentAction);
     QAction *separator = new QAction(this);
     separator->setSeparator(true);
-    d->editor->insertAction(NULL, separator);
-    d->editor->insertAction(NULL, d->editCutAction);
-    d->editor->insertAction(NULL, d->editCopyAction);
-    d->editor->insertAction(NULL, d->editCopyReferencesAction);
-    d->editor->insertAction(NULL, d->editPasteAction);
-    d->editor->insertAction(NULL, d->editDeleteAction);
+    d->editor->addAction(separator);
+    d->editor->addAction(d->editCutAction);
+    d->editor->addAction(d->editCopyAction);
+    d->editor->addAction(d->editCopyReferencesAction);
+    d->editor->addAction(d->editPasteAction);
+    d->editor->addAction(d->editDeleteAction);
+    separator = new QAction(this);
+    separator->setSeparator(true);
+    d->editor->addAction(separator);
 
     // TODO
 
     connect(d->editor, SIGNAL(selectedElementsChanged()), this, SLOT(updateActions()));
-    updateActions();
-
-    fitActionSettings();
 
     setXMLFile(RCFileName);
+
+    new FindDuplicatesUI(this, d->editor);
+    d->lyx = new LyX(this, d->editor);
+    connect(d->editor, SIGNAL(selectedElementsChanged()), d->lyx, SLOT(updateActions()));
+
+    d->colorLabelContextMenu = new ColorLabelContextMenu(d->editor);
+
+    updateActions();
+    fitActionSettings();
 }
 
 bool KBibTeXPart::saveFile()
@@ -484,7 +502,7 @@ void KBibTeXPart::elementViewDocument()
     if (!d->viewDocumentMenu->actions().isEmpty()) {
         // TODO: Choose "best" URL instead of first (local file over remote, PDF over website, ...)
         QAction *action = d->viewDocumentMenu->actions().first();
-        QDesktopServices::openUrl(KUrl(action->text())); // TODO KDE way?
+        QDesktopServices::openUrl(KUrl(action->data().toString())); // TODO KDE way?
     }
 }
 
@@ -501,12 +519,9 @@ void KBibTeXPart::fitActionSettings()
 
 bool KBibTeXPart::openFile()
 {
-    kDebug() << "Opening file: " << url();
-
     setObjectName("KBibTeXPart::KBibTeXPart for " + url().pathOrUrl());
 
     FileImporter *importer = d->fileImporterFactory(url());
-
     importer->showImportDialog(widget());
 
     qApp->setOverrideCursor(Qt::WaitCursor);
@@ -518,13 +533,12 @@ bool KBibTeXPart::openFile()
     delete importer;
 
     if (bibtexFile == NULL) {
-        kWarning() << "Opening file failed: " << url();
+        kWarning() << "Opening file failed:" << url().pathOrUrl();
         qApp->restoreOverrideCursor();
         return false;
-    } else
-        kDebug() << "File contains " << bibtexFile->count() << " entries";
+    }
 
-    bibtexFile->setProperty(File::Url, url());
+    bibtexFile->setProperty(File::Url, QUrl(url()));
 
     d->model->setBibTeXFile(bibtexFile);
     d->editor->setModel(d->model);
@@ -600,12 +614,27 @@ void KBibTeXPart::updateActions()
     d->elementEditAction->setEnabled(!emptySelection);
     d->editCopyAction->setEnabled(!emptySelection);
     d->editCopyReferencesAction->setEnabled(!emptySelection);
-    d->editCutAction->setEnabled(!emptySelection);
-    d->editDeleteAction->setEnabled(!emptySelection);
+    d->editCutAction->setEnabled(!emptySelection && isReadWrite());
+    d->editDeleteAction->setEnabled(!emptySelection && isReadWrite());
 
     int numDocumentsToView = d->updateViewDocumentMenu();
     /// enable menu item only if there is at least one document to view
     d->elementViewDocumentAction->setEnabled(!emptySelection && numDocumentsToView > 0);
     /// activate sub-menu only if there are at least two documents to view
     d->elementViewDocumentAction->setMenu(numDocumentsToView > 1 ? d->viewDocumentMenu : NULL);
+    d->elementViewDocumentAction->setToolTip(numDocumentsToView == 1 ? d->viewDocumentMenu->actions().first()->text() : QLatin1String(""));
+
+    /// update list of references which can be sent to LyX
+    QStringList references;
+    if (d->editor->selectionModel() != NULL) {
+        QModelIndexList mil = d->editor->selectionModel()->selectedRows();
+        for (QModelIndexList::ConstIterator it = mil.constBegin(); it != mil.constEnd(); ++it) {
+            Entry *entry = dynamic_cast<Entry*>(d->editor->bibTeXModel()->element(d->editor->sortFilterProxyModel()->mapToSource(*it).row()));
+            if (entry != NULL)
+                references << entry->id();
+        }
+    }
+    d->lyx->setReferences(references);
+
+    d->colorLabelContextMenu->setEnabled(isReadWrite());
 }
